﻿//------------------------------------------------------------------
//	COM ポート名を ComboBox で使うためのクラス
//	このプログラムは下記を参考にした
// http://truthfullscore.hatenablog.com/entry/2014/01/10/180608
//
//		作成者：三上直樹，2018/05/13 作成，©三上直樹 2018
//------------------------------------------------------------------

using System;
using System.Linq;
using System.Windows.Forms;

using System.Management;	// ManagementClass, ManagementObject 用

namespace ComPortComboBox
{
	public static class ComPort
	{
		// ComboBox の Items の内容を作成し，一番大きな番号のポートを
		// ComboBox の Text に表示する
		public static void UpdateComboBoxItems(ComboBox comboBox)
		{
			string[] ports = GetComPortNames();
			if (ports == null) return;

			ports = ports.OrderBy
					( x => int.Parse( x.Substring(x.IndexOf("(COM")+4)
									   .TrimEnd(")".ToCharArray()) )  )
					.ToArray();
			comboBox.Items.Clear();
			comboBox.Items.AddRange(ports);
			comboBox.SelectedIndex = comboBox.Items.Count - 1;
			if (comboBox.SelectedIndex == -1) comboBox.Text = "";
		}

	    // COM ポートのリストの作成
		public static string[] GetComPortNames()
		{
			ManagementClass ports = new ManagementClass("Win32_SerialPort");
			var nameList = ports.GetInstances()
								.OfType<ManagementObject>()
								.Select(x => x.GetPropertyValue("Name"))
								.Where(x => x != null)
								.Select(x => x.ToString())
								.Where(x => String.IsNullOrEmpty(x) || x.Contains("(COM"));
			return nameList.ToArray();
		}

		// シリアル通信用デバイス名から COM ポート番号を抽出
		//		戻り値： "COMxxx", xxx は番号
		public static string ExtractComPort(string str)
		{
			return str.Substring(str.IndexOf("(COM")+1)
					  .TrimEnd(")".ToCharArray());
		}
	}
}
