﻿//--------------------------------------------------------------
//	いろいろなメソッド
//
//		作成者：三上直樹，2020/02/11 作成，©三上直樹 2020
//--------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Numerics;
using ComPortComboBox;

using Mikami;
using static System.Math;

namespace F446_GraphicEqualizer
{
	public partial class Form1 : Form
	{
		// シリアルポートのオープン処理
		private bool SerialPortOpen()
        {
            serialPort1.BaudRate = BAUD_RATE_;

			// ComboBox1 で選択された COM ポートを使うようにする
			string port = ComPort.ExtractComPort(comboBox1.Text);
			if (!comboBox1.Text.Contains("STLink") || (port == ""))	return false;

            serialPort1.PortName = port;
			try
			{
				serialPort1.Open();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
				return false;
			}

			return true;
		}

		// シリアルポートのクローズ処理
		private void SerialPortClose()
        {
            serialPort1.Close();
        }

		// シリアルポートに文字列を送る
		private void SendTo(string str)
		{
			try
			{
				serialPort1.Write(str + "\r");
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		// USB ポートの監視．変更があった場合は，タイマを起動し，そこで
		// ComboBox の COM ポートに関する表を更新する
		protected override void WndProc(ref Message m)
		{
		    if (m.Msg == 0x0219)	// WM_DEVICECHANGE
			{
				int winMsg = m.WParam.ToInt32();
				if ( (winMsg == 0x8000) || (winMsg == 0x8004) )
					timer1.Enabled = true;	// timer1 を有効化
			}
			base.WndProc(ref m);
		}

		// グラフィック･イコライザ全体の周波数応答を取得
		public Complex GetResponse(Complex u)
		{
			Complex hz = 1;
			for (int n=0; n<BANDS_; n++)
				hz = hz*filters_[n].GetResponse(u);
			return hz;
		}

		// グラフィック･イコライザで使うフィルタの定義
		private void GrEqDefinition()
		{
			for (int n=0; n<BANDS_; n++)
			{
				BiquadGrEq.Type t = BiquadGrEq.Type.PEAK;
				if (n == 0)			t = BiquadGrEq.Type.LSHL;
				if (n == BANDS_-1)	t = BiquadGrEq.Type.HSHL;
				filters_[n] = new BiquadGrEq(F0_MIN_*Pow(2, n), Q_VAL_, t, SAMPLING_FRQ_);
			}
		}

		// ブースト/カット量の取得
		private void MyScroll(object sender, EventArgs e)
		{
			int n = (int)((TrackBar)sender).Tag;	// TrackBar の番号を取得
			float db = bar_[n].Value/100.0f;
			filters_[n].SetCoefficients(db);
			pictureBox1.Invalidate();

			// 送信するデータのフォーマット
			// 最初の文字：TrackBar の番号（int型に対応）
			// 以降の文字：dB 値（float型に対応）
			string str = n.ToString("0") + db.ToString("0.##");
			SendTo(str);
		}

		// TrackBar 生成
		private void TrackBarGerator()
		{
			for (int n=0; n<BANDS_; n++)
			{
				bar_[n] = new TrackBar();
				bar_[n].Enabled = false;
				bar_[n].Location = new Point(pictureBox1.Left + Dpi(n*49-8), Dpi(206));
				bar_[n].Size = new Size(Dpi(42), Dpi(104));
				bar_[n].Maximum = 1000;		// +10 dB に対応
				bar_[n].Minimum = -1000;	// -10 dB に対応
				bar_[n].Orientation = Orientation.Vertical;
				bar_[n].Tag = n;
				bar_[n].TickFrequency = bar_[n].Maximum;
				bar_[n].TickStyle = TickStyle.TopLeft;
				bar_[n].Value = 0;
				bar_[n].Scroll += MyScroll;
				Controls.Add(bar_[n]);
			}
		}

		// Debug 時のみ例外のメッセージを出すようにするために使用
		private void ExectionChatch(Exception ex)
		{
			#if DEBUG
			MessageBox.Show(ex.Message + "," + ex.Source + "," +
							ex.TargetSite.ToString());
			#endif
		}
	}
}
