﻿//--------------------------------------------------------------
//	いろいろな使うメソッド
//
//		作成者：三上直樹，2020/02/13 作成，©三上直樹 2020
//--------------------------------------------------------------

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Numerics;
using ComPortComboBox;

namespace F446_LPF_HPF_Variable
{
	public partial class Form1 : Form
	{
		// シリアルポートのオープン処理
		private bool SerialPortOpen()
        {
            serialPort1.BaudRate = BAUD_RATE_;

			// ComboBox1 で選択された COM ポートを使うようにする
			string port = ComPort.ExtractComPort(comboBox1.Text);
			if (!comboBox1.Text.Contains("STLink") || (port == ""))	return false;

            serialPort1.PortName = port;
			try
			{
				serialPort1.Open();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
				return false;
			}

			return true;
		}

		// シリアルポートのクローズ処理
		private void SerialPortClose()
        {
            serialPort1.Close();
        }

		// シリアルポートに文字列を送る
		private void SendTo(string str)
		{
			try
			{
				serialPort1.Write(str + "\r");
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		// USB ポートの監視．変更があった場合は，タイマを起動し，そこで
		// ComboBox の COM ポートに関する表を更新する
		protected override void WndProc(ref Message m)
		{
		    if (m.Msg == 0x0219)	// WM_DEVICECHANGE
			{
				int winMsg = m.WParam.ToInt32();
				if ( (winMsg == 0x8000) || (winMsg == 0x8004) )
					timer1.Enabled = true;	// timer1 を有効化
			}
			base.WndProc(ref m);
		}

		// FIR フィルタの再設計と周波数特性の表示
		private void RedesignDisplay()
		{
			iir_.Design(ORDER_, pb_, SAMPLING_FRQ_, fC_);
			pictureBox1.Invalidate();
		}

		// float 型引数 x の整数部分のみを変換した文字列の先頭に
		// 文字列 str を付加して送信する
		private void SendInt(string str, float x)
		{
			SendTo(str + ((int)x).ToString());
		}

		// Debug 時のみ例外のメッセージを出すようにするために使用
		private void ExectionChatch(Exception ex)
		{
			#if DEBUG
			MessageBox.Show(ex.Message + "," + ex.Source + "," +
							ex.TargetSite.ToString());
			#endif
		}
	}
}
