﻿//--------------------------------------------------------------
//	スペクトログラムで使うメソッド
//
//		作成者：三上直樹，2020/02/11 作成，©三上直樹 2020
//--------------------------------------------------------------

using System;
using System.Windows.Forms;
using Mikami.ComPortComboBox;

namespace F446_Spectrogram
{
	public partial class Form1 : Form
	{
		// シリアルポートのオープン処理
		private bool serialPortOpen()
        {
			serialPort1.BaudRate = BAUD_RATE_;

			// ComboBox1 で選択された COM ポートを使うようにする
			string port = ComPort.ExtractComPort(comboBox1.Text);
			if (!comboBox1.Text.Contains("STLink") || (port == ""))	return false;

            serialPort1.PortName = port;
			try
			{
				serialPort1.Open();
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
				return false;
			}

			return true;
        }

		// シリアルポートに文字列を送る
		private void SendTo(string str)
		{
			try
			{
				serialPort1.Write(str + "\r");
			}
			catch (Exception ex)
			{
				ExectionChatch(ex);
			}
		}

		// データの転送を要求する
		private void RequestXfer(string str)
		{
			rxData_ = "";		// バッファのクリア
			if (str == "GO")
			{
				// dB 計算の際のシフト量
				str += Convert.ToChar((trackBar1.Value + 20) + 0x20);
				
				// 高域強調の有無
				str += radioButton1.Checked ? "Y" : "N";

				// 解析方法（FFT/線形予測法）
				str += radioButton3.Checked ? "F" : "L"; 
			}
			SendTo(str);
		}

		// USB ポートの監視．変更があった場合は，タイマを起動し，その tick イベントハンドラで
		// ComboBox の COM ポートに関する表を更新する
		protected override void WndProc(ref Message m)
		{
		    if (m.Msg == 0x0219)	// WM_DEVICECHANGE
			{
				int winMsg = m.WParam.ToInt32();
				if ( (winMsg == 0x8000) || (winMsg == 0x8004) )
					timer1.Enabled = true;	// timer1 を有効化して，timer1_Tick() の中で
											// ComPort.UpdateComboBoxItems() を実行している
			}
			base.WndProc(ref m);
		}

		// 0 ～ 10,000 の範囲の数を 2 文字で表したコードを float 型に変換する
		// 10,000 を 1.0f に対応させる
		private float C10000ToFloat(string str)
		{
			//int x = (((int)str[0] - 0x20) << 6) + (int)str[1] - 0x20;
			uint x = ((uint)str[0] - 0x10)*100 + (uint)str[1] - 0x10;
			#if DEBUG
				if (x > 10000) MessageBox.Show(x.ToString());
			#endif
			return x/10000.0f;
		}

		// Debug 時のみ例外のメッセージを出すようにするために使用
		private void ExectionChatch(Exception ex)
		{
			#if DEBUG
			MessageBox.Show(ex.Message + "," + ex.Source + "," +
							ex.TargetSite.ToString());
			#endif
		}
	}
}
